/*  XLS XOR password cracker
    Copyright (C) 2003-7  Nick Lamb <njl+xlcrack@filter.tlrmx.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <string.h>
#include <stdio.h>
#include <gsf/gsf.h>

#include <gsf/gsf-utils.h>
#include <gsf/gsf-input-stdio.h>
#include <gsf/gsf-infile.h>
#include <gsf/gsf-infile-msole.h>

#include <langinfo.h>

typedef struct match_struct {
  int flags;
  guint16 type;
  int offset;
  guint8 bitmap;
} match;

#define BIFF_ONLY 0x1
#define OLE_ONLY  0x2

const match blanks[]=
 {
   { 0, 0x5, 8, 0xfe},  /* BOOLERR */

/* OpenOffice.org documentation is wrong, bit 0x02 can be set */
   { BIFF_ONLY, 0x6, 15, 0xfc}, /* BIFF FORMULA */

/* OpenOffice.org documentation is wrong? Check which bits can be set */
   { OLE_ONLY, 0x6, 14, 0x80}, /* OLE FORMULA */
   { OLE_ONLY, 0x6, 15, 0xff},

   { 0, 0x8, 3, 0xff}, /* ROW */
   { 0, 0x8, 5, 0xfe},
   { 0, 0x8, 10, 0xfe},

   { 0, 0xe, 0, 0xfe},  /* PRECISION */
   { 0, 0xe, 1, 0xff},

   { 0, 0xf, 0, 0xfe},  /* REFMODE */
   { 0, 0xf, 1, 0xff},

   { 0, 0x11, 0, 0xfe}, /* ITERATION */
   { 0, 0x11, 1, 0xff},

   { 0, 0x12, 0, 0xfe}, /* PROTECT */
   { 0, 0x12, 1, 0xff},

   { 0, 0x19, 0, 0xfe}, /* WINDOWPROTECT */
   { 0, 0x19, 1, 0xff},

   { 0, 0x1d, 4, 0xff}, /* SELECTION */

/* OpenOffice.org documentation is wrong, bit 0x02 can be set */
   { 0, 0x21, 6, 0xfc}, /* ARRAY */

   { 0, 0x22, 0, 0xfe}, /* DATEMODE */
   { 0, 0x22, 1, 0xff},

   { 0, 0x2a, 0, 0xfe}, /* PRINTHEADERS */
   { 0, 0x2a, 1, 0xff},

   { 0, 0x2b, 0, 0xfe}, /* PRINTGRIDLINES */
   { 0, 0x2b, 1, 0xff},

   { 0, 0x31, 3, 0xff},

   { 0, 0x40, 0, 0xfe}, /* BACKUP */
   { 0, 0x40, 1, 0xff},

   { 0, 0x5f, 0, 0xfe}, /* SAVERECALC */
   { 0, 0x5f, 1, 0xff},

   { 0, 0x63, 0, 0xfe}, /* OBJECTPROTECT */
   { 0, 0x63, 1, 0xff},

#if 0
   { 0, 0x7d, 1, 0xff}, /* COLINFO */
   { 0, 0x7d, 3, 0xff},
#endif

   { 0, 0x82, 0, 0xfe}, /* GRIDSHEET */
   { 0, 0x82, 1, 0xff},

   { 0, 0x83, 0, 0xfe}, /* HCENTER */
   { 0, 0x83, 1, 0xff},

   { 0, 0x84, 0, 0xfe}, /* VCENTER */
   { 0, 0x84, 1, 0xff},

   { 0, 0x85, 4, 0xf0}, /* BOUNDSHEET */
   { 0, 0x85, 5, 0xf0},

   { 0, 0x8d, 0, 0xfc}, /* HIDEOBJ */
   { 0, 0x8d, 1, 0xff},

   { 0, 0x90, 1, 0xf8}, /* SORT */

   { 0, 0x91, 1, 0xff}, /* */

   { 0, 0x92, 1, 0xff}, /* PALETTE */

   { 0, 0xda, 0, 0xfe}, /* BOOKBOOL */
   { 0, 0xda, 1, 0xff},

   { 0, 0xdd, 0, 0xfe}, /* SCENPROTECT */
   { 0, 0xdd, 1, 0xff},

   { 0, 0x161, 0, 0xfe}, /* DSF */
   { 0, 0x161, 1, 0xff},

   { 0, 0x1b1, 0, 0xf0}, /* CF */
   { 0, 0x1b1, 1, 0xf0},

   { 0, 0x1be, 0, 0x08}, /* DV */
   { 0, 0x1be, 1, 0xf8},
   { 0, 0x1be, 2, 0x03},
   { 0, 0x1be, 3, 0xff},

   { 0, 0x203, 3, 0xff}, /* NUMBER */

   { 0, 0x205, 3, 0xff}, /* BOOLERR */
   { 0, 0x205, 7, 0xfe},

/* OpenOffice.org documentation is wrong? Check which bits can be set */
   { 0, 0x206, 14, 0xf0}, /* FORMULA */
   { 0, 0x206, 15, 0xff},

   { 0, 0x208, 3, 0xff}, /* ROW */
   { 0, 0x208, 5, 0xfe},

   { 0, 0x221, 6, 0xfc}, /* ARRAY */
   { 0, 0x221, 7, 0xff},

   { 0, 0x225, 1, 0xff}, /* DEFAULTROWHEIGHT */
   { 0, 0x225, 0, 0xf0},

   { 0, 0x27e, 3, 0xff}, /* RK */

/* OpenOffice.org documentation is wrong? Check which bits can be set */
   { 0, 0x406, 14, 0xf0}, /* FORMULA */
   { 0, 0x406, 15, 0xff}, /* FORMULA */
};

char *cp1252_to_ascii[256] = {
 "*","*","*","*","*","*","*","*","*","*","*","*","*","*","*","*",
 "*","*","*","*","*","*","*","*","*","*","*","*","*","*","*","*",
 " ","!","\"","#","$","%","&","'","(",")","*","+",",","-",".","/",
 "0","1","2","3","4","5","6","7","8","9",":",";","<","=",">","?",
 "@","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O",
 "P","Q","R","S","T","U","V","W","X","Y","Z","[","\\","]","^","_",
 "`","a","b","c","d","e","f","g","h","i","j","k","l","m","n","o",
 "p","q","r","s","t","u","v","w","x","y","z","{","|","}","~","*",
 "*","*","*","*","*","*","*","*","*","*","*","*","*","*","*","*",
 "*","*","*","*","*","*","*","*","*","*","*","*","*","*","*","*",
 "*","*","*","*","*","*","*","*","*","*","*","*","*","*","*","*",
 "*","*","*","*","*","*","*","*","*","*","*","*","*","*","*","*",
 "*","*","*","*","*","*","*","*","*","*","*","*","*","*","*","*",
 "*","*","*","*","*","*","*","*","*","*","*","*","*","*","*","*",
 "*","*","*","*","*","*","*","*","*","*","*","*","*","*","*","*",
};

char *cp1252_to_utf8[256] = {
 "�","�","�","�","�","�","�","�","�","�","�","�","�","�","�","�",
 "�","�","�","�","�","�","�","�","�","�","�","�","�","�","�","�",
 " ","!","\"","#","$","%","&","'","(",")","*","+",",","-",".","/",
 "0","1","2","3","4","5","6","7","8","9",":",";","<","=",">","?",
 "@","A","B","C","D","E","F","G","H","I","J","K","L","M","N","O",
 "P","Q","R","S","T","U","V","W","X","Y","Z","[","\\","]","^","_",
 "`","a","b","c","d","e","f","g","h","i","j","k","l","m","n","o",
 "p","q","r","s","t","u","v","w","x","y","z","{","|","}","~","�",
 "€","�","‚","ƒ","„","…","†","‡","ˆ","‰","Š","‹","Œ","�","Ž","�",
 "�","‘","’","“","”","•","–","—","˜","™","š","›","œ","�","ž","Ÿ",
 "�","¡","¢","£","¤","¥","¦","§","¨","©","ª","«","¬","�","®","¯",
 "°","±","²","³","´","µ","¶","·","¸","¹","º","»","¼","½","¾","¿",
 "À","Á","Â","Ã","Ä","Å","Æ","Ç","È","É","Ê","Ë","Ì","Í","Î","Ï",
 "Ð","Ñ","Ò","Ó","Ô","Õ","Ö","×","Ø","Ù","Ú","Û","Ü","Ý","Þ","ß",
 "à","á","â","ã","ä","å","æ","ç","è","é","ê","ë","ì","í","î","ï", 
 "ð","ñ","ò","ó","ô","õ","ö","÷","ø","ù","ú","û","ü","ý","þ","ÿ",
};

char *codeset;
char **cp1252 = cp1252_to_ascii;

const int num_blanks = sizeof(blanks) / sizeof(match);

guint8 byte_index[32], byte_rotate[32], byte_mask[32];

guint16 hash, key;
guint8 low, high; /* bytes of key below */

/* 128 bits of recovered key sequence */
guint8 set[16];
guint8 key_seq[16];

#define CRYPTO_TYPE_NONE 0
#define CRYPTO_TYPE_XOR 1
#define CRYPTO_TYPE_RC4 2
#define CRYPTO_TYPE_NEWER 3

/* global status information */
int known_length, crypto_type;
gboolean file_is_ole;
int quit;

static void
reset_everything(void)
{
  known_length = 0;
  crypto_type = CRYPTO_TYPE_NONE;
  file_is_ole = 0;
  quit = 0;

  int k;
  for (k= 0; k < 16; ++k) {
    set[k] = 0;
    key_seq[k] = 0;
  }
}

static guint16
ms_biff_password_key(const unsigned char *password, int length)
{
  guint16 key = 0, key_base= 0x8000, key_final= 0xffff;
  int index;

  for (index = 0; index < length; ++index) {
    guint8 chr = password[(length - 1) - index] & 0x7f;
    int bit_index;

    for (bit_index = 0; bit_index < 8; ++bit_index) {
      key_base = (key_base << 1) | (key_base >> 15);
      if (key_base & 1) key_base ^= 0x1020;
      key_final = (key_final << 1) | (key_final >> 15);
      if (key_final & 1) key_final ^= 0x1020;
      if (chr & 1) key ^= key_base;
      chr = chr >> 1;
    }
  }
  key = key ^ key_final;
  
  return key;
}

static guint16
ms_biff_password_hash(const unsigned char *password, int length)
{
  int tmp, index= 0;
  guint16 chr, hash= 0;

  do {
    chr = password[index];
    index++;
    tmp = (chr << index);
    hash ^= (tmp & 0x7fff) | (tmp >> 15);
  } while (index < length);
  hash = hash ^ length ^ 0xce4b;

  return hash;
}

/* WARNING - don't use set_and_forget once unswizzle has been called */

static void
set_and_forget(int offset, int n, guint8 bitmap, const guint8 *buffer)
{
  int index = (offset + n) % 16;
  guint8 bits = (buffer[n] << 3) | (buffer[n] >> 5);

  if (crypto_type == CRYPTO_TYPE_RC4) return;
  if (crypto_type == CRYPTO_TYPE_NEWER) return;

  if (crypto_type == CRYPTO_TYPE_NONE) {
    bitmap = (bitmap << 3) | (bitmap >> 5);
  }

  if ((key_seq[index] & set[index] & bitmap)
          != (bits & set[index] & bitmap)) {
    printf("WARNING! multi-key problem?\n");
    printf("Please report this to the program author, thanks.\n");
    set[index]= set[index] | bitmap;
    key_seq[index]= key_seq[index] | (bits & bitmap);
  } else {
    set[index]= set[index] | bitmap;
    key_seq[index]= key_seq[index] | (bits & bitmap);
  }
}

/* Unlike set_and_forget, set_byte can be called after unswizzle */

static void
set_byte(int index, guint8 byte)
{
  if ((key_seq[index] & set[index]) != (byte & set[index])) {
    printf("WARNING! key detection problem\n");
    printf("Please report this to the program author, thanks.\n");
  }
  set[index] = 0xff;
  key_seq[index] = byte;
}

static void
unswizzle(guint8 low, guint8 high)
{
  int k;

  for (k= 0; k < 16; k+=2) {
    key_seq[k] = (key_seq[k] >> 2) | (key_seq[k] << 6);
    set[k] = (set[k] >> 2) | (set[k] << 6);
    key_seq[k] ^= low;

    key_seq[k+1] = (key_seq[k+1] >> 2) | (key_seq[k+1] << 6);
    set[k+1] = (set[k+1] >> 2) | (set[k+1] << 6);
    key_seq[k+1] ^= high;
  }
}

static void
get_more_bits()
{
  guint8 seed[15] = {0xbb, 0xff, 0xff, 0xba, 0xff, 0xff, 0xb9, 0x80,
                      0x00, 0xbe, 0x0f, 0x00, 0xbf, 0x0f, 0x00 };

  /* Only works on real passwords */
  if (crypto_type != CRYPTO_TYPE_XOR) return;


  /* Re-arrange key sequence data to our satisfaction */
  unswizzle(low, high);

  /* Spare bytes after password ends ? */
  if (set[15] == 0xff) {
    int k, offset;

    switch (key_seq[15]) {
    case 0x00:
      if (set[13] == 0xff && key_seq[13] == 0xbf)
        offset=1;
      else if (set[13] == 0xff && key_seq[13] == 0xbe)
        offset=4;
      else if (set[13] == 0xff && key_seq[13] == 0xb9)
        offset=7;
      else
        offset= 16;
      break;
    case 0x0f:
      if (set[14] == 0xff && key_seq[14] == 0xbe)
        offset=5;
      else if (set[14] == 0xff && key_seq[14] == 0x00)
        offset=2;
      else
        offset= 16;
      break;
    case 0x80:
      offset= 8; break;
    case 0xb9:
      offset= 9; break;
    case 0xba:
      offset= 12; break;
    case 0xbb:
      offset= 15; break;
    case 0xbe:
      offset= 6; break;
    case 0xbf:
      offset= 3; break;
    case 0xff:
      if (set[14] == 0xff && key_seq[14] == 0xba)
        offset=11;
      else if (set[14] == 0xff && key_seq[14] == 0xbb)
        offset=14;
      else if (set[13] == 0xff && key_seq[13] == 0xba)
        offset=10;
      else if (set[13] == 0xff && key_seq[13] == 0xbb)
        offset=13;
      else
        offset= 16;
      break;
    default:
      printf("WARNING! key suffix problem\n");
      offset= 16; break;
    }

    if (offset == 16) {
      offset = known_length ? known_length : 16;
    } else if (known_length == 0) {
      known_length = offset;
    } else if (known_length != offset) {
      printf("WARNING! problem determining password length\n");
    }

    for (k = 0; offset + k < 16; ++k) {
      set_byte(offset + k, seed[k]);
    }
  }
}

static gboolean
test_pw_seq(guint8 *seq)
{
  int k;

  if (known_length && ms_biff_password_hash(seq, known_length) == hash
                   && ms_biff_password_key(seq, known_length) == key) {
    seq[known_length] = 0;
    return TRUE;
  }

  for (k= 1; k < 16; ++k) {
    if (ms_biff_password_hash(seq, k) == hash
     && ms_biff_password_key(seq, k) == key) {
      seq[k] = 0;
      return TRUE;
    }
  }

  return FALSE;
}

static void
print_key_data(guint8 *seq)
{
  int k, count= 0;

  for (k= 0; k < 128; ++k) {
    if (set[k / 8] & (1 << (k % 8))) {
      count++;
    }
  }
  printf("%3d bits ", count);

  for (k= 0; k < 16; ++k) {
    if (set[k] == 0xff)
      printf("%02x ", seq[k]);
    else if ((set[k] & 0xf0) == 0xf0)
      printf("%x* ", seq[k] >> 4);
    else if ((set[k] & 0x0f) == 0x0f)
      printf("*%x ", seq[k] & 0x0f);
    else
      printf("** ");
  }
  printf(" ");

  for (k= 0; k < 15; ++k) {
    if (set[k] == 0)
      printf("_");
    else
      printf("%s", cp1252[seq[k]]);
  }
  printf("\n");
}

static void
print_password(guint8 *seg)
{
  for (; *seg != 0; seg++) {
    printf("%s", cp1252[*seg]);
  }
}

/* do not call test_many_keys when more than 32 bits are unknown */

static void
set_mask(guint8 *seq, int index, int rotate, int mask)
{
  guint8 byte = seq[index];
  seq[index] = (byte & mask) + (1 << rotate);
}

static void
unset_mask(guint8 *seq, int index, int rotate, int mask)
{
  guint8 byte = seq[index];
  seq[index] = (byte & mask);
}

static gboolean
test_many_keys(guint8 *seq)
{
  int k, n, max, missing = 0;

  for (k= 0; k < 128; ++k) {
    if ((set[k / 8] & (1 << (k % 8))) == 0) {
      byte_index[missing] = k / 8;
      byte_rotate[missing] = k % 8;
      byte_mask[missing] = 0xff ^ (1 << (k % 8));
      missing++;
    }
  }

  max= 1 << missing;

  for (k= 0; k < max; ++k) {
    for (n = 0; n < missing; ++n) {
      if (k & (1 << n)) {
        set_mask(seq, byte_index[n], byte_rotate[n], byte_mask[n]);
      } else {
        unset_mask(seq, byte_index[n], byte_rotate[n], byte_mask[n]);
      }
    }
    if (test_pw_seq(seq)) {
      return TRUE;
    }
  }

  return FALSE;
}

static gboolean
pw_from_seq(guint8 *seq)
{
  int k, count= 0;

  for (k= 0; k < 128; ++k) {
    if (set[k / 8] & (1 << (k % 8))) {
      count++;
    }
  }

  if (count == 128) {
    if (test_pw_seq(seq)) return TRUE;
  } else if (count >= 112) {
    /* up to 16 bits to guess, probably get a good guess */
    if (test_many_keys(seq)) return TRUE;
  }
  
  return FALSE;
}

static void biff_stats(GsfInput *stream)
{
  gsf_off_t pos = gsf_input_tell(stream);
  int k, offset;
  guint16 type, length;
  const guint8 *buffer;

  buffer= gsf_input_read(stream, 4, NULL);
  if (buffer == NULL) {
    quit= 1;
    return;
  }

  type = GSF_LE_GET_GUINT16(buffer);
  length = GSF_LE_GET_GUINT16(buffer + 2);
  offset = (pos + length + 4) % 16;
  
  buffer= gsf_input_read(stream, length, NULL);
  if (length && buffer == NULL) {
    quit= 1;
    return;
  }

  if (type == 0x2f) {
    if (length == 4) {
      crypto_type = CRYPTO_TYPE_XOR;
      low = GSF_LE_GET_GUINT8(buffer);
      high = GSF_LE_GET_GUINT8(buffer + 1);
      key = (high << 8) + low;
      hash = GSF_LE_GET_GUINT16(buffer + 2);
    } else if (length == 6) {
      crypto_type = CRYPTO_TYPE_XOR;
      low = GSF_LE_GET_GUINT8(buffer + 2);
      high = GSF_LE_GET_GUINT8(buffer + 3);
      key = (high << 8) + low;
      hash = GSF_LE_GET_GUINT16(buffer + 4);
    } else if (length == 0x36 && buffer[4] == 1) {
      crypto_type = CRYPTO_TYPE_RC4;
    } else {
      crypto_type = CRYPTO_TYPE_NEWER;
    }
  }

  for (k= 0; k < num_blanks; ++k) {
    if (type == blanks[k].type) {
      if ((blanks[k].flags & OLE_ONLY) && !file_is_ole) continue;
      if ((blanks[k].flags & BIFF_ONLY) && file_is_ole) continue;
      set_and_forget(offset, blanks[k].offset, blanks[k].bitmap, buffer);
    }
  }
}

static int try_file(char *filename)
{
  GsfInput *input;
  GsfInfile *file;
  GsfInput *stream = NULL;
  GError *err = NULL;
  int i= 0;

  /* Stolen from Gnumeric's XL plugin */
  static char const * const content[] =
   { "Workbook", "WORKBOOK", "workbook", "Book", "BOOK", "book" };

  reset_everything();

  gsf_init();
  input = GSF_INPUT(gsf_input_stdio_new(filename, &err));
  if (input == NULL) {
    fprintf(stderr, "Error opening file \"%s\"\n", filename);
    return 2;
  }

  file = GSF_INFILE(gsf_infile_msole_new(input, &err));
  if (file) {
    file_is_ole = TRUE;
    /* OLE stream, open the workbook inside */

    do {
      stream = gsf_infile_child_by_name (file, content[i++]);
    } while (stream == NULL && i < G_N_ELEMENTS (content));
    if (stream == NULL) {
      fprintf(stderr, "Error opening Workbook \"%s\"\n", filename);
      return 4;
    }
  } else {
    gsf_input_seek (input, 0, G_SEEK_SET);
    stream = input;
  }

  while (!gsf_input_eof(stream) && (quit == 0)) {
    biff_stats(stream);
  }

  get_more_bits();
  
  switch (crypto_type) {
    case CRYPTO_TYPE_XOR:
      if (pw_from_seq(key_seq)) {
        if (!strcmp((char *) key_seq, "VelvetSweatshop")) {
          printf("%s: has no password, see documentation\n", filename);
        } else {
          printf("%s: password is '", filename);
          print_password(key_seq);
          printf("'\n");
        }
      } else {
        printf("%s: password recovery failed, see below for clues\n", filename);
        print_key_data(key_seq);
      }
      break;
    case CRYPTO_TYPE_NONE:
      printf("%s: is not encrypted\n", filename);
#ifdef DEBUG_BLANKS
      print_key_data(key_seq);
#endif
      break;
  
    case CRYPTO_TYPE_RC4:
      printf("%s: uses strong cryptography\n", filename);
      printf("This tool only works on XOR cryptography (e.g. Excel 95)\n");
      break;

    case CRYPTO_TYPE_NEWER:
      printf("%s: uses very strong cryptography\n", filename);
      printf("This tool only works on XOR cryptography (e.g. Excel 95)\n");
  }

  return 0;
}

int main(int argc, char *argv[])
{
  if (argc < 2) {
    fprintf(stderr, "Usage: %s filename.xls ...\n", argv[0]);
    return 1;
  }

  codeset = nl_langinfo(CODESET);
  if (strcasecmp(codeset, "UTF-8")) {
    cp1252 = cp1252_to_utf8;
  }

  int k;
  for (k = 1; k < argc; ++k) {
    try_file(argv[k]);
  }

  return 0;
}
